/*HEADER------------------------------------------------------------------------
|
| PROGRAM:  create_greg_rpt.sas (macro)
||
| PURPOSE:  This macro creates report using output from the macro %distributed_regression 
                        for linear and logistic regressions (generalized linear models). 
                        It can also be used to generate similar reports using output from macros lin_reg_with_data_input,
 						logit_reg_with_data_input. The later macros use standard
                        SAS regression procedure on combined dataset and generate output datasets that follow 
                        naming conventions of the datasets generated by  %distributed_regression (up to a prefix).
                    |
|
| MACRO PARAMETERS:
|
	 INPUT:    
          lib_in                       The name of the library that has output datasets from regression.
                                          For DRA it is always msoc library. Must be specified.

         rpt_dir_out			   The name of the output  directory for the report

         prefix_r					  Prefix used for dataset created by regression macro.
                                          For example, &RunID_.
       
	    regr_type_cd             Type of the regression. 1-linear. 2-logistic.
|	     
		distributed_reg_in		This affects the name of the report. When distributed_reg_in=1
                                           the report name includes word Distributed.

     rpt_format						File format for the report. Can be PDF or RTF.
                                          The default is PDF

      pdf_style                     Determines ODS style used with the report.
                                         Default value produces calor report. For black and
                                         white report use style=JOURNAL,JOURNAL2  
                                          Can be used not only with PDF but with RTF
                                          output as well.

      format_N                     Numeric format used in printed tables.
                                          Default is D11.3.

    Output:                          Report in pdf or rtf format in the directory specified by
                                          by the parameter rpt_dir_out.
                                          For example, dr1_Linear_Distributed_reg_rpt.rtf

   	USAGE EXAMPLES:
 
			* For linear;						 		
			 %create_greg_rpt( lib_in=msoc
                                   ,rpt_dir_out=&dplocal.
                                   ,prefix_r=dr1_
                                   ,regr_type_cd=1
                                   ,distributed_reg_in=1
								   ,pdf_style=JOURNAL2
								    ,rpt_format=rtf
								     );

* For logistic;
                %create_greg_rpt( lib_in=msoc
                                   ,rpt_dir_out=&dplocal.
                                   ,prefix_r=dr1_
                                   ,regr_type_cd=2
                                   ,distributed_reg_in=1
								   ,pdf_style=DEFAULT
								    ,rpt_format=rtf
								     );
 
	First version:  02/02/2017, Yury Vilk. 
+------------------------------------------------------------------------HEADER*/	 
 %MACRO create_greg_rpt( lib_in=msoc
                                         , rpt_dir_out=./
                                         ,prefix_r=
                                        , regr_type_cd=1
                                        , distributed_reg_in=1
									    ,pdf_style=DEFAULT
									   ,bradiusmax=2%
									    ,bradiusmin=0.5%
									   ,additional_title=
									   ,format_N=D11.3
									   ,rpt_format=pdf
                                         );

 %LOCAL regr_type_dsc distributed_reg_dsc _vars_for_modelfit 
               _indep_vars _indep_vars_sh  _indep_vars_numb  _auc_
			    graph_grp dash min_Mean max_Mean __attrid_option
        ;


 %LET regr_type_dsc=Linear;
 %LET distributed_reg_dsc=Distributed;
  %LET graph_grp=%STR(group=dp_cd);
  %LET _vars_for_modelfit=Criterion Value ;
  %LET dash=_;

   /* For buble graph*/
  %LET  __attrid_option=;

  %LET NOTOC=NOTOC;

 %MACRO wrapper;

	  %IF                &regr_type_cd.=1 %THEN %LET regr_type_dsc=Linear;
	 %ELSE 	%IF  &regr_type_cd.=2 %THEN %LET regr_type_dsc=Logistic;
	  %ELSE %IF  &regr_type_cd.=3 %THEN %LET regr_type_dsc=Poisson;

%IF   &regr_type_cd.=2  %THEN %LET _vars_for_modelfit=Criterion Value ;
%ELSE                                         %LET _vars_for_modelfit=Criterion DF Value ValueDF;

%IF &distributed_reg_in.=0 %THEN   
				   %DO;
       %LET distributed_reg_dsc=;
	 /*  %LET graph_grp=; */
	   %LET dash=;
					%END;

	   	/* Optimize bubble graph for black and white printer when ODS style is JOURNAL, JORNAL2 ...	*/

	 %IF %SUBSTR(&pdf_style., 1,5)=JOURN %THEN 	%LET __attrid_option=%STR(ATTRID=BW);

	%IF &rpt_format.=rtf %THEN %LET NOTOC=;
 %MEND;

 %wrapper;

 %LET title1=&regr_type_dsc. &distributed_reg_dsc. Regression. &additional_title.;
 %PUT title1=&title1.;
 %PUT __attrid_option=&__attrid_option.;
	
	DATA _NULL_;
		LENGTH _indep_vars $ 32000;
		 RETAIN _indep_vars;
		   SET &lib_in..&prefix_r.p_est_HC(KEEP=Variable) END=last;
			 _indep_vars=CATX(" ",_indep_vars,Variable);

			 IF _N_=10 THEN 	CALL SYMPUT("_indep_vars_sh", STRIP( _indep_vars));

			 IF last THEN  
			                    DO;
               CALL SYMPUT("_indep_vars", STRIP( _indep_vars));
			   CALL SYMPUT("_indep_vars_numb", STRIP( PUT(_N_,8.)));
			  IF _N_ < 10 THEN 	CALL SYMPUT("_indep_vars_sh", STRIP( _indep_vars));
			                   END;

		RUN;

		%PUT _indep_vars=&_indep_vars.	;
		%PUT _indep_vars_sh=&_indep_vars_sh.	;
		%PUT _indep_vars_numb=&_indep_vars_numb.	;
   
  	 ODS GRAPHICS ON /  RESET=INDEX 
                                         IMAGENAME="&prefix_r.D_Reg" /*To see default image names use ODS TRACE */
                                         BORDER=OFF
										 IMAGEFMT=PNG
                                        ;

	   OPTIONS DEVICE=PNG;

	 ODS LISTING CLOSE	;

 ODS &rpt_format.  DPI=400 FILE="&rpt_dir_out.&prefix_r.&regr_type_dsc._&distributed_reg_dsc.&dash.reg_rpt.&rpt_format." 
						      &NOTOC. STYLE=&pdf_style. NOGTITLE;

     *OPTIONS ORIENTATION=LANDSCAPE;
	 OPTIONS ORIENTATION=PORTRAIT;
		OPTIONS MISSING=" ";

	  TITLE1 &title1;


%IF  &regr_type_cd.=1 %THEN 
                    %DO;
	  TITLE2 "Analysis of Variance";
	  TITLE3 "Dependent Variable: #BYVAL(Dependent)"	;
	  	OPTIONS NOBYLINE ;
	
       PROC PRINT DATA=&lib_in..&prefix_r.ANOVA(DROP=MODEL)  Noobs LABEL;
	    BY 	Dependent;
	    LABEL SS="Sum of Squares"               
	                 MS="Mean Square"
	                 FValue="F Value"
	                 ProbF="Pr > F"
				;
	   RUN;	

      TITLE2; 
	  TITLE3;


	 	OPTIONS BYLINE ;
 	
	  DATA 	 &prefix_r.FitStatistics_rp1(KEEP=Label1 nValue1  RENAME=(Label1=Statistic nValue1 =Value))
				  &prefix_r.FitStatistics_rp2(KEEP=Label2 nValue2  RENAME=(Label2=Statistic nValue2 =Value))
                    ;
	    SET &lib_in..&prefix_r.FitStatistics(DROP=MODEL Dependent cValue1 cValue2);
		
		    IF Label1 NE " " THEN OUTPUT &prefix_r.FitStatistics_rp1;
		    IF Label2 NE " " THEN OUTPUT &prefix_r.FitStatistics_rp2;
	 RUN;
	  
  PROC TRANSPOSE DATA=&lib_in..&prefix_r.model_coeff(KEEP=_AIC_ _BIC_ _SBC_) 
                                        OUT=&prefix_r.FitStatistics_rp3(RENAME=(_LABEL_=Statistic COL1=Value));

  RUN;

  DATA 	 &prefix_r.FitStatistics_rp4;
  LENGTH Statistic $ 50;
    SET  &prefix_r.FitStatistics_rp1 &prefix_r.FitStatistics_rp2 &prefix_r.FitStatistics_rp3(IN=in3);
	IF in3 THEN Statistic=CATX(" ",_NAME_,Statistic);
	KEEP  Statistic Value;
 RUN;

	ODS STARTPAGE=NO;
	
	 PROC REPORT DATA=  &prefix_r.FitStatistics_rp4 ;
		   COLUMN  ("Fit Statistics" (Statistic Value) );
			  DEFINE Statistic/ DISPLAY;
		LABEL Statistic ="Statistic" ;
		RUN;

 	 %END;
 
	 %ELSE 
	               %DO;
	 
TITLE2; 

		PROC REPORT DATA=&lib_in..&prefix_r.Convrg_Status ;
		   COLUMN  Status_Dsc Iteration Reason;
			  DEFINE Status_Dsc/DISPLAY;
			LABEL Reason="Convergence Message";
		RUN;

		 ODS STARTPAGE=NO;
	
		PROC REPORT DATA= &lib_in..&prefix_r.Iter_Parms_Hist ;
		   COLUMN  ("Iteration history" (Iteration  Neg2LogLike &_indep_vars_sh.) );
			  DEFINE Iteration/ORDER ID;
			LABEL  Neg2LogLike="-2 Log Likelihood";
				FORMAT Neg2LogLike &_indep_vars_sh. &format_N. ;  
		RUN;

	 PROC REPORT DATA=  &lib_in..&prefix_r.modelfit ;
		   COLUMN  ("Fit Statistics" (&_vars_for_modelfit.) );
			  DEFINE Criterion/ DISPLAY;
			  	FORMAT Value  &format_N. ;
		RUN;

	 ODS STARTPAGE=NO;

	 PROC REPORT DATA=  &lib_in..&prefix_r.GLOB_NULL_CHISQ ;
		   COLUMN  ("Testing Global Null Hypothesis: BETA=0" (Test ChiSq DF ProbChiSq) );
			  DEFINE Test/ DISPLAY;
			  WHERE UPCASE(Test)="LIKELIHOOD RATIO";
			FORMAT ChiSq  &format_N. ;
		RUN;

			       %END;

		ODS STARTPAGE=YES;

		TITLE2 ;
		 FOOTNOTE1 ;

	PROC REPORT DATA=  &lib_in..&prefix_r.P_EST_HC(RENAME=(StdErr=StdErr1 Probt=P_Value));
	 COLUMN  ("Parameter Estimates" (Variable DF Estimate StdErr1 /*tValue*/ P_Value 
                              LowerCL UpperCL HCStdErr  /*HCTValue*/ 
				              HCProbt HCLowerCL HCUpperCL ) );
			  DEFINE Variable/ DISPLAY ID;
		FORMAT Estimate StdErr1 LowerCL UpperCL HCStdErr HCLowerCL HCUpperCL &format_N.;
				   ;
		RUN;

		ODS STARTPAGE=NO;

		PROC REPORT DATA=  &lib_in..&prefix_r.COV_EST(OBS=10);
	        COLUMN  ("Covariance of Estimates. First 10 columns shown." (Variable  &_indep_vars_sh.) );
			  DEFINE Variable/ DISPLAY ID;	
           FORMAT _NUMERIC_ &format_N.; 
		RUN;

		PROC REPORT DATA=  &lib_in..&prefix_r.HC_COV(OBS=10);
	        COLUMN  ("Robust Covariance of Estimates. First 10 columns shown." (Variable  &_indep_vars_sh.) );
			  DEFINE Variable/ DISPLAY ID;	
           FORMAT _NUMERIC_ &format_N.; 
		RUN;

		%IF  &regr_type_cd.=2 %THEN 
                    %DO;

		 /* 20171127. Limited calculation of HL statistic to standard test only Test_type=1;*/

		PROC REPORT DATA=  &lib_in..&prefix_r.HL_ChiSq MISSING;
	        COLUMN  ("Hosmer and Lemeshow Goodness-of-Fit Test." 
                            (/*dp_cd */ ChiSq  df ValueDF ProbChiSq) );
			 /* DEFINE dp_cd/ ORDER; */
             LABEL 	ChiSq="Chi-Square" df="DF"  ProbChiSq="Pr > ChiSq"  ;
         	FORMAT ChiSq  &format_N.  ProbChiSq PVALUE6.4;	
			WHERE Test_type=1;
		RUN;

		PROC SQL NOPRINT;
		  SELECT STRIP(PUT(MAX(_AUC_), D8.3)) INTO :_auc_
		     FROM  &lib_in..&prefix_r.roc
			 ;
		QUIT;

		%PUT _auc_=&_auc_.;

		ODS STARTPAGE=YES;
	
	PROC SGPLOT DATA=&lib_in..&prefix_r.roc
								CYCLEATTRS ASPECT=1	 
								;
		TITLE2 "ROC Curve for Model";
		TITLE3	"Area Under Curve =&_auc_.";

	     SERIES X=_1MSPEC_  Y=_SENSIT_ /LINEATTRS=(THICKNESS=2 PX);
		   LINEPARM X=0 Y=0 SLOPE=1/LEGENDLABEL="Reference Line"  
                                                                  LINEATTRS=(THICKNESS=2 PX); 
	
	  *	REFLINE   1  /AXIS=Y; 
		
		XAXIS MIN=-0.05 MAX=1.05;
		YAXIS MIN=-0.05 MAX=1.05;
   RUN;

   TITLE2 " ";
   TITLE3 " ";
							   %END;
		
	%IF     (&regr_type_cd.=1 AND &_indep_vars_numb. <=10)  %THEN
	                          %DO;
     PROC REPORT DATA=  &lib_in..&prefix_r.CollinDiag(DROP=MODEL 	Dependent);
	        COLUMN  ("Collinearity Diagnostics. Possible collinearity: for high condition index (>10) two variance proportion values greater than 0.5." 
                                (_ALL_) 
                              );
			  DEFINE 	Number/ ORDER;	
           FORMAT _NUMERIC_ &format_N.; 
		RUN;
							   %END;

	
 PROC SQL;
   CREATE TABLE _all_dp_cd AS
     SELECT DISTINCT dp_cd
	   FROM   &lib_in..&prefix_r.RESID_SUM_BY_PCT
	   ORDER BY dp_cd
          ;
	QUIT;

 data __attrmap_2;
    SET _all_dp_cd;
  id='BW'; value=dp_cd;
	 /*For first 3 dp_cd set colors that look clearly different when printed on black and white printer. 
       This will be used with black and with ODS style like JOURNAL */

   IF _N_ =1 THEN DO;fillcolor='black'; output; END;
   IF  _N_ =2 THEN DO; fillcolor='grey'; output; END;
   IF  _N_ =3 THEN DO; fillcolor='white'; output; END;
				/*Use default colors for other values*/
  run;
 
	DATA &prefix_r.RESID_SUM_BY_PCT_g	;
	   SET &lib_in..&prefix_r.RESID_SUM_BY_PCT;

	 	            /*Model based StdErr for mean */
		    
		   _StdErr_M=SQRT(_ADJ_VARIANCE__Mean/_FREQ_);
		   _RESIDUAL__Mean_norm= _RESIDUAL__Mean/ _StdErr_M;
		   Resid_sq_Model_var_ratio=_RESIDUAL_SQ__Mean/_ADJ_VARIANCE__Mean;

		   IF dp_cd >0;

		LABEL _MEAN__Mean="Mean of Predicted Response"
				   _RESP__Mean="Mean of Observed Response"
 				    _RESIDUAL__Mean="Mean of Residual "
				   _RESIDUAL__Mean_norm="Normalized Mean of Residual"
				   _RESIDUAL_SQ__Mean="Mean of Residual Square"
				   _ADJ_VARIANCE__Mean="Mean of  Model Variance"
				   	 Resid_sq_Model_var_ratio="Ratio of Residual sq. to Model Variance"
	
			  ;
	RUN;

	PROC SORT DATA=&prefix_r.RESID_SUM_BY_PCT_g;
    	BY _MEAN__Mean ;
	RUN;

	 PROC SQL NOPRINT;
 	   SELECT  PUT(MIN(MIN(_MEAN__Mean,_RESP__Mean)) ,2.) INTO :min_Mean
	   		FROM  &prefix_r.RESID_SUM_BY_PCT_g
			;
	   SELECT  PUT(CEIL(MAX(MAX(_MEAN__Mean,_RESP__Mean)) ) ,2.) INTO :max_Mean
	   		FROM  &prefix_r.RESID_SUM_BY_PCT_g
			;
	  QUIT;

	  %PUT min_Mean=&min_Mean.;
	  %PUT max_Mean=&max_Mean.;

	 
    PROC SGPLOT DATA=&prefix_r.RESID_SUM_BY_PCT_g	
								CYCLEATTRS	ASPECT=1  DATTRMAP=__attrmap_2
								;
	    TITLE2 "Mean of observed response vs. mean of predicted response.";
		TITLE3 "The data are grouped by percentiles of predicted values."  ;
        TITLE4 "The radius of a bubble is proportional to a number of observations in a group.";

   BUBBLE X=_MEAN__Mean Y=_RESP__Mean SIZE= _FREQ_ / &graph_grp.
                                                                                                          BRADIUSMAX=&BRADIUSMAX.
																										&__attrid_option.  
                                            ;																				
	   LINEPARM X=0 Y=0 SLOPE=1/ LEGENDLABEL="Reference Line" 
                                                           LINEATTRS=(THICKNESS=2 PX); 

	   	XAXIS MIN=&min_Mean. MAX=&max_Mean.;
		YAXIS MIN=&min_Mean. MAX=&max_Mean.;

   RUN;

    TITLE2 ;
    TITLE3 ;
    TITLE4;

	ODS STARTPAGE=NO;

    PROC SGPLOT DATA=&prefix_r.RESID_SUM_BY_PCT_g
								CYCLEATTRS	 DATTRMAP=__attrmap_2
								;
	* TITLE2 "Normalized mean of residual vs. mean of predicted response.";
	TITLE2 "Mean of residual vs. mean of predicted response.";
	TITLE3 "The data are grouped by percentiles of predicted values."  ;
    TITLE4 "The radius of a bubble is proportional to a number of observations in a group.";

	BUBBLE X=_MEAN__Mean Y=_RESIDUAL__Mean SIZE= _FREQ_ /&graph_grp. 
                                                                                                                      BRADIUSMAX=&BRADIUSMAX.
																													&__attrid_option.
																										              ;

	 * BUBBLE X=_MEAN__Mean Y=_RESIDUAL__Mean_norm SIZE= _FREQ_ /&graph_grp. 
                                                                                                                      BRADIUSMAX=&BRADIUSMAX.
				 ;
	  	REFLINE   0  /AXIS=Y LINEATTRS=(THICKNESS=2 PX); 
   RUN;

	 TITLE2 ;
    TITLE3 ;
    TITLE4;

	/*
    PROC SGPLOT DATA=&prefix_r.RESID_SUM_BY_PCT_g	
								CYCLEATTRS	ASPECT=1  DATTRMAP=__attrmap_2
								;
		TITLE2 "Mean of  residual square compared with mean of model variance.";
		TITLE3 "The data are grouped by percentiles of predicted values."  ;
        TITLE4 "The radius of a bubble is proportional to a number of observations in a group.";

         BUBBLE X=_MEAN__Mean Y=_RESIDUAL_SQ__Mean SIZE= _FREQ_ /&graph_grp. 
                                                                                                                          BRADIUSMAX=&BRADIUSMAX.
																										                  &__attrid_option.
                                            ;
         PBSPLINE X=_MEAN__Mean Y=_ADJ_VARIANCE__Mean/DEGREE=3  
																								    LINEATTRS=(COLOR=BLACK PATTERN=1 THICKNESS=2 PX) 
																								    NOMARKERS
																									LEGENDLABEL="Model Variance"  
                                    ;
   RUN;

    TITLE2 ;
    TITLE3 ;
    TITLE4;
	  */
		 ODS &rpt_format. CLOSE;

        ODS LISTING;

	%MEND;	



	







	



	      